/*
 * seccomp-bpf/hello-lib.c --
 *
 *      This version of the program does the same as hello.c but using
 *      the libseccomp library providing an easy to use, platform
 *      independent, interface to the Linux Kernel's syscall filtering
 *      mechanism.
 */

#define _POSIX_C_SOURCE 200809L

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <seccomp.h>
#include "util.h"

static void install_filter(void)
{
    int rc;
    scmp_filter_ctx ctx;

    /* Create a default deny filter disallowing all system calls and
     * then allow all system calls required. */

    ctx = seccomp_init(SCMP_ACT_KILL);
    if (ctx == NULL) {
        fatal("seccomp_init() failed\n");
    }

    /* Add rules for allowed system calls. We allow the write system
     * call writing to the standard output and exit_group system call
     * used to terminate the process. */

    rc = seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(write), 1,
                          SCMP_A0(SCMP_CMP_EQ, STDOUT_FILENO));
    if (rc < 0) {
        fatal("seccomp_rule_add() failed: %d\n", rc);
    }

    rc = seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(exit_group), 0);
    if (rc < 0) {
        fatal("seccomp_rule_add() failed: %d\n", rc);
    }

    /* Generate the seccomp BPF filter and load it into the kernel. */

    rc = seccomp_load(ctx);
    if (rc < 0) {
        fatal("seccomp_load() failed: %d\n", rc);
    }
}

int main(void)
{
    fputs("hello ", stdout);            /* triggers library initializations */
    install_filter();
    fputs("world!\n", stdout);
    fputs("process dying (invalid write system call)\n", stderr);
    return EXIT_SUCCESS;
}
